%
% Demo in order to comparing the richness of extraction with differents 
% feature extraction methods: PCA, PLS, and all kernels involved in the 
% SS-KPLS and SS-KOPLS. To do that, computes the overall accuracy OA and 
% kappa index using a linear classifier for several number of features 
% extract with the differents methods.
%
% Emma Izquierdo-Verdiguier, 2012
% emma.izquierdo@uv.es
% 

clear;clc;close all;

warning('off')

fontname = 'Arial';
fontsize = 14;
fontunits = 'points';
set(0,'DefaultAxesFontName',fontname,'DefaultAxesFontSize',fontsize,'DefaultAxesFontUnits',fontunits,...
    'DefaultTextFontName',fontname,'DefaultTextFontSize',fontsize,'DefaultTextFontUnits',fontunits,...
    'DefaultLineLineWidth',5,'DefaultLineMarkerSize',8,'DefaultLineColor',[0 0 0]);
%% Path
addpath(genpath('./methods'))
addpath(genpath('./tools'))

% Settings
Nl = 8;            % labeled
Nu = 400;          % unlabeled
Nfmax = 8;        % maximum number of features to be extracted
rand('seed',4444)
randn('seed',4444)

disp('Step 1: Selecting data ...')
% Load data
[Xt,Yt]=generate_toydata(Nl+Nu,'threeclassproblem');
Xt=scalestd(Xt);
disp('Done!')

disp('Step 2: Selecting labeled and unlabeled data ...')
% Selecting labeled and unlabeled data
X=[];Y=[];Xu=[];Yu=[];
for t=1:length(unique(Yt))
    ind=find(Yt==t);
    r=randperm(length(ind));
    sel=ind(r(1:Nl));
    X=[X;Xt(sel,:)];
    Y=[Y;Yt(sel)];
    sel=ind(r(Nl+1:Nu+Nl));
    Xu=[Xu;Xt(sel,:)];
    Yu=[Yu;Yt(sel)];
end

disp('Done!')
figure_projections(X,Xu,Y,Yu)
xlabel('Feature 1')
ylabel('Feature 2')
title('Data')
axis off
legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')

%% PCA
disp('Step 3: Training PCA ...')
[OA_pca,Kappa_pca]=Assesspca(X,Xu,Y,Yu,Nfmax);
disp('Done!')

%% PLS
disp('Step 4: Training PLS ...')
[OA_pls,Kappa_pls]=Assesspls(X,Xu,Y,Yu,Nfmax);
disp('Done!')

%% Compute train and test bag kernels
N = 20;  % #realizations of gmm
k = 2:20; % number of clusters

[Kbag,Centroides,IDX] = bagkernelgmm(X,Xu,N,k);
KbagUL = bagkernelImGMM(X,Xu,Centroides,IDX,N,k);

%fix weight parameter: K = alphas*Ks+(1-alphas)*Kc.
alphas = 0.5;

%% SS-KPLS
[OA,Kappa]=Assesskpls(X,Xu,Y,Yu,Kbag,KbagUL,Centroides,IDX,alphas,Nfmax);

%% SS-KOPLS
[OA_kopls,Kappa_kopls]=Assesskopls(X,Xu,Y,Yu,Kbag,KbagUL,Centroides,IDX,alphas);

% %% Figures
figure, plot(OA_pca,'c','LineWidth',3)
hold on, plot(OA_pls,'k','LineWidth',3)
hold on, plot(OA(1,:),'b','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(OA(3,:),'r','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(OA(2,:),'g','LineWidth',5,'Marker','o','MarkerSize',8)
hold on, plot(OA_kopls(1,:),'b--','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(OA_kopls(3,:),'r--','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(OA_kopls(2,:),'g--','LineWidth',5,'Marker','o','MarkerSize',8)
legend('PCA','PLS','Cluster Kernel, KcPLS','KPLS, Ks','SS-KPLS, K','Cluster Kernel, KcOPLS','KOPLS, Ks','SS-KOPLS, K')
set(legend,'Location','SouthEast');
xlabel('# features','fontsize',24)
ylabel('OA(%)','fontsize',24)
grid

figure, plot(Kappa_pca,'c','LineWidth',3)
hold on, plot(Kappa_pls,'k','LineWidth',3)
hold on, plot(Kappa(1,:),'b','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(Kappa(3,:),'r','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(Kappa(2,:),'g','LineWidth',5,'Marker','o','MarkerSize',8)
hold on, plot(Kappa_kopls(1,:),'b--','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(Kappa_kopls(3,:),'r--','LineWidth',4,'Marker','.','MarkerSize',24)
hold on, plot(Kappa_kopls(2,:),'g--','LineWidth',5,'Marker','o','MarkerSize',8)
legend('PCA','PLS','Cluster Kernel, KcPLS','KPLS, Ks','SS-KPLS, K','Cluster Kernel, KcOPLS','KOPLS, Ks','SS-KOPLS, K')
set(legend,'Location','SouthEast');
xlabel('# features','fontsize',24)
ylabel('Kappa','fontsize',24)
grid

