% Computes the overall accuracy OA and kappa index using a linear
% classifier. The estimated labels are obtained by projected data onto
% principal components of PCA method.
%
% Inputs:
%       -X    : Original labeled data. Matrix, M(samples)xF(features).
%       -Xu   : Original unlabeled data.  Matrix, N(samples)xF(features).
%       -Y    : Labels of X.  Vector, M(samples)x1. (It's only use to predict.)
%       -Yu   : Labels of Xu. Vector, M(samples)x1. (It's only use to testing.)
%       -Nfmax: # features extracted.
%
% Outputs:
%       -OA   : Overall acuracy. Vector, 1XF(features).
%       -Kappa: Estimated Cohen's Kappa coefficient. Vector, 1XF(features).



function [OA,Kappa]=Assesspca(X,Xu,Y,Yu,Nfmax)

%-----------------
%  PCA
%-----------------
r=randperm(size(Xu,1));
Xp=[X;Xu(r(1:(end-size(Xu,1)/4)),:)];
Xtest=Xu(r((end-size(Xu,1)/4):end),:);

Yp=[Y;Yu(r(1:(end-size(Xu,1)/4)),:)];
Yp1=binarize(Yp);
Ytest=Yu(r((end-size(Xu,1)/4):end),:);

npmax = min([Nfmax,size(Xp,2)]);
Upca = pca(Xp,npmax);

figure,
for t=1:min(Nfmax,size(Upca.basis,2))
    Xpca=Xp*Upca.basis(:,1:t);
    Xpca2=Xtest*Upca.basis(:,1:t);

    if t==1||t==2
        subplot(1,2,t),plotFeatures(X,Xu,Upca.basis,[],Y,Yu,t)
        title(strcat('PCA, component ',num2str(t)))
    end
    
    Xpca1=[Xpca ones(size(Xpca,1),1)];
    W=pinv(Xpca1)*Yp1;
    Ypred=Xpca2*W(1:end-1,:)+repmat(W(end,:),size(Xpca2,1),1);
    [x Ypca]=max(Ypred,[],2);
    
    RESpca=assessment(Ytest,Ypca,'class');
    OA(t)=RESpca.OA;
    Kappa(t)=RESpca.Kappa;
end

Xpca=Xp*Upca.basis(:,1:2);
Xpca2=Xtest*Upca.basis(:,1:2);

figure_projections(Xpca,Xpca2,Yp,Ytest)
xlabel('Feature 1')
ylabel('Feature 2')
axis off
title('Projected data onto the first 2 directions of PCA')
legend('Test data, class 1','Test data, class 2','Train data, class 1','Train data, class 2')