% Computes the overall accuracy OA and kappa index using a linear
% classifier. The estimated labels are obtained by projected data onto
% principal components of PLS method.
%
% Inputs:
%       -X    : Original labeled data. Matrix, M(samples)xF(features).
%       -Xu   : Original unlabeled data. Matrix, N(samples)xF(features).
%       -Y    : Labels of X.  Vector, M(samples)x1.
%       -Yu   : Labels of Xu. Vector, M(samples)x1. (It's only use to testing.)
%       -Nfmax: # features extracted.
%
% Outputs:
%       -OA   : Overall acuracy. Vector, 1XF(features).
%       -Kappa: Estimated Cohen's Kappa coefficient. Vector, 1XF(features).

function [OA,Kappa]=Assesspls(X,Xu,Y,Yu,Nfmax)

%-----------------
%  PLS
%-----------------

% % Center labels
Yb = binarize(Y); Yb = Yb - ones(length(Y),1)*mean(Yb);

npmax = min([Nfmax,size(X,2),max(Y)]);
U=plsSB(X,Y,npmax);

figure,
for t=1:min(Nfmax,size(U.basis,2))
    Xpls=X*U.basis(:,1:t);
    Xpls2=Xu*U.basis(:,1:t);
    
    if t==1||t==2
        subplot(1,2,t),plotFeatures(X,Xu,U.basis,[],Y,Yu,t)
        title(strcat('PLS, component ',num2str(t)))
    end
    Xpls1=[Xpls ones(size(Xpls,1),1)];
    W=pinv(Xpls1)*Yb;
    Ypred=Xpls2*W(1:end-1,:)+repmat(W(end,:),size(Xpls2,1),1);
    [x Ypls]=max(Ypred,[],2);

    RESpls=assessment(Yu,Ypls,'class');
    OA(t)=RESpls.OA;
    Kappa(t)=RESpls.Kappa;
end

Xpls=X*U.basis(:,1:2);
Xpls2=Xu*U.basis(:,1:2);

figure_projections(Xpls,Xpls2,Y,Yu)
xlabel('Feature 1')
ylabel('Feature 2')
axis off
title('Projected data onto the first 2 directions of PLS')
legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')
