% Builds a GMM test cluster kernel given input train and test data
% Inputs:
%       - Xtrain     : Original train data. Matrix,   M(samples)xF(features).
%       - Xtest      : Original test data. Matrix, N(samples)xF(features).
%       - C_tot      : Strut containing centers and mixing components of the Gaussians.
%       - IDX_tot    : Strut containing partitions data determined by GMM.
%       - N          : # iterations of GMM.
%       - Kvector    : # maximum cluster of GMM
%
% Outputs:
%       - KbagUL    : Test cluster kernel. Matrix, M(train samples)xN(test samples).


function KbagUL = bagkernelImGMM(Xtrain,Xtest,C_tot,IDX_tot,N,Kvector)

KbagUL = zeros(size(Xtrain,1),size(Xtest,1));

nk=0;
for k=Kvector
    nk=nk+1;
    for n=1:N
        if ~isempty(C_tot{nk,n})
            % Posterior probability of components
            Pgmm = posterior(C_tot{nk,n},Xtrain);
            % Compute posterior probability and partitions data of test
            % data respect to train data
            [ID_UL PP]= closerClusterGMM(Xtest,C_tot{nk,n});
            idx_tot=IDX_tot{nk,n};
            idx_tot=idx_tot(1:size(Xtrain,1),:);
            % Test bag kernel from GMM clustering
            for p=1:length(ID_UL)
                for q=1:length(idx_tot)
                    KbagUL(q,p) = KbagUL(q,p) + PP(p,:)*Pgmm(q,:)';
                end
            end
        end
    end
end

%-------------------------------------------------------------------------
% Normalization of test cluster kernel
%-------------------------------------------------------------------------
KbagUL = KbagUL/max(max(KbagUL));