% Generates toy classification problems

function [X,Y]=generate_toydata(n,method)

% [X,Y]=generate_toydata(n,method)
%
% Generates toy classification problems.
%
% INPUTS:
%
%    n     : points in class +1 and n points in class 2
%
%    method:
%       'lines'            : Parallel lines
%       'xor'              : The classical XOR gate
%       'noisyxor'         : The classical XOR gate with added Gaussian noise
%       'balls3'           : Three ball classes
%       'parabolloid'      : One parabolloid
%       'ellipsoids'       : Parallel ellipsoids
%       'ellipsoids3'      : Three ellipsoids
%       'moons'            : Two moons
%       'swiss'            : 2d swiss roll 
%       'threeclassproblem':simple problem with 3 classes
%
% OUTPUTS:
%     
%    X: Generated data (samples in rows, features in columns)
%    Y: Column vector with labels for each data point
%
% Gustavo Camps-Valls, 2011(c)
% gcamps@uv.es
%

[X,Y]=feval(method,n);

function [X,Y]=lines(n);

        var = 1;
        x = 0.015;
        y = 0.015;
        X1=[randn(n,1)*0.01+x       randn(n,1)*0.01+y];
        Y1=ones(n,1);
        X2=[randn(n,1)*0.01-x       randn(n,1)*0.01-y];
        Y2=2*ones(n,1);
        X=[X1;X2];
        Y=[Y1;Y2];

function [X,Y]=xor(n);

        n = n/2;
        X1 = mvnrnd([1;20],[1 0;0 1],n);
        Y1 = ones(n,1);
        X2 = mvnrnd([1;1],[1 0;0 1],n);
        Y2 = 2*ones(n,1);
        X3 = mvnrnd([20;1],[1 0;0 1],n);
        Y3 = ones(n,1);
        X4 = mvnrnd([20;20],[1 0;0 1],n);
        Y4 = 2*ones(n,1);
        X  = [X1;X2;X3;X4];
        Y  = [Y1;Y2;Y3;Y4];

function [X,Y]=noisyxor(n);

        n = n/2;
        X1 = mvnrnd([1;10],[.1 0;0 1],n);
        Y1 = ones(n,1);
        X2 = mvnrnd([1;1],[.1 0;0 1],n);
        Y2 = 2*ones(n,1);
        X3 = mvnrnd([10;1],[.1 0;0 1],n);
        Y3 = ones(n,1);
        X4 = mvnrnd([10;10],[.1 0;0 1],n);
        Y4 = 2*ones(n,1);
        X  = [X1;X2;X3;X4];
        Y  = [Y1;Y2;Y3;Y4];

function [X,Y]=balls(n);

        X1 = mvnrnd([15 15],[4 0;0 4],n);
        Y1 = ones(n,1);
        X2 = mvnrnd([10;10],[3 0;0 3],n);
        Y2 = 2*ones(n,1);
        X  = [X1;X2];
        Y  = [Y1;Y2];

function [X,Y]=balls3(n);

        n = round(n/3);
        X1 = mvnrnd([15 15],[1 0;0 1],n);
        Y1 = ones(n,1);
        X2 = mvnrnd([10;10],[1 0;0 1],n);
        Y2 = 2*ones(n,1);
        X3 = mvnrnd([15;1],[1 0;0 1],n);
        Y3 = 3*ones(n,1);
        X  = [X1;X2;X3];
        Y  = [Y1;Y2;Y3];

function [X,Y]=parabolloid(n);

        space=1.5;
        noise = 0.05;
        r=randn(n,1)*noise+1;
        theta=randn(n,1)*pi;
        r1=1.1*r; r2=r;
        X=([r1.*cos(theta) abs(r2.*sin(theta))]);
        Y=ones(n,1);

function [X,Y]=ellipsoids(n)
		
        mean1 = [-1;0];
        mean2 = [1;0];
        cov = [1 .6; .6 1];
        
		X1 = mvnrnd(mean1,cov,n);
		X2 = mvnrnd(mean2,cov,n);
		
		X = [X1;X2];
		media = mean(X);
		X = X - ones(2*n,1)*mean(X);
		Y = zeros(2*n,1); 
        Y(1:n,1) = 1; 
        Y(n+1:end,1) = 2;
		
function [X,Y]=ellipsoids3(n)
		
        mean1 = [-2;1];
        mean2 = [2;-2];
        mean3 = [4;2];
        cov = [1 .9; .9 1];
        
		X1 = mvnrnd(mean1,cov,n);
		X2 = mvnrnd(mean2,cov,n);
		X3 = mvnrnd(mean3,cov,n);

        Y1 = ones(n,1);
        Y2 = 2*ones(n,1);
        Y3 = 3*ones(n,1);

		X = [X1;X2;X3];
		Y = [Y1;Y2;Y3];

function [X,Y]=moons(n)

        space = 2;
        noise = 0.1;
        r=randn(n,1)*noise+1;
        theta=randn(n,1)*pi;
        r1=1.1*r; r2=r;
        X1=([r1.*cos(theta) abs(r2.*sin(theta))]);
        Y1=ones(n,1);

        r=randn(n,1)*noise+1;
        theta=randn(n,1)*pi+2*pi;
        r1=1.1*r; r2=r;
        X2=([r1.*cos(theta)+space*rand -abs(r2.*sin(theta)) + 0.6 ]);
        Y2=2*ones(n,1);

        X=[X1;X2];
        Y=[Y1;Y2];

function [X,Y]=swiss(n);

        %GENERATE SAMPLED DATA 1
        tt = (3*pi/2)*(1+2*rand(1,n));  %height = 21*randn(1,N);
        X1 = [tt.*cos(tt); tt.*sin(tt)]';

        % GENERATE SAMPLED DATA 2
        tt = (3*pi/2)*(1+2*rand(1,n));  %height = 21*randn(1,N);
        X2 = [tt.*cos(tt);  tt.*sin(tt)]'; X2=1.6*X2;
        X=[X1;X2]/25+randn(2*n,2)*0.03;
        Y=[ones(n,1); 2*ones(n,1)];

function [X,Y] = aviris(n)

    load IndianPines.mat
    X = reshape(Xtotal,145*145,220);
    Y = Ytotal(:);
    nozero = find(Y~=0);
    X = X(nozero,1:20:end);
    Y = Y(nozero,:);
    t = length(Y)
    r = randperm(t);
    X = X(r(1:2*n),:);
    Y = Y(r(1:2*n),:);
    
    function [X,Y] = threeclassproblem(n)
        
        space = 2;
        noise = 0.1;
        r=randn(n,1)*noise+1;
        theta=randn(n,1)*pi;
        r1=1.1*r; r2=r;
        X1=([r1.*cos(theta) abs(r2.*sin(theta))]);
        Y1=ones(n,1);

        r=randn(n,1)*noise+1;
        theta=randn(n,1)*pi+2*pi;
        r1=1.1*r; r2=r;
        X2=([r1.*cos(theta) abs(r2.*sin(theta)) + 0.55 ]);
        Y2=2*ones(n,1);
        
        space = 2;
        r=randn(n,1)*noise+1;
        theta=randn(n,1)*pi+2*pi;
        r1=1.1*r; r2=r;
        X3=([r1.*cos(theta) abs(r2.*sin(theta))-0.55]);
        Y3=3*ones(n,1);

        X=[X1;X2;X3];
        Y=[Y1;Y2;Y3];
    
