% Fast computation of the kernel matrix

% To speed up the computation, we exploit a decomposition of the Euclidean distance (norm)
%
% Inputs: 
%	ker: 'rbf'
%	X:	data matrix with training samples in rows and features in columns
%	X2:	data matrix with test samples in rows and features in columns
%	sigma: width of the RBF kernel
% 	b: bias in the linear and polinomial kernel 
%	d: degree in the polynomial kernel
%
% Output:
%	K: kernel matrix
%
% Gustavo Camps-Valls
% 2006(c)
%

function K = kernelmatrix(ker,X,X2,sigma);

% clear all;
% clear functions;
% clc;
% t = cputime;
% ker = 'rbf'
% X = randn(200,1000); K = zeros(size(X,2));
% X2 = [];
% sigma = 10;
% b = 1;
% d = 5;
% 

if  strcmp(ker,'lin')
	
	if ~isempty(X2)
	  K = sigma*X'*X2;
	else
	  K = sigma*X'*X;
	end;
	
elseif  strcmp(ker,'poly')
	
	if ~isempty(X2)
	  K = (X' * X2 + b).^d;
	else
	  K = (X' * X + b).^d;
	end;

elseif strcmp(ker,'rbf')
	
	n1sq = sum(X.^2);
	n1 = size(X,2);
	
	if isempty(X2);
	  D = (ones(n1,1)*n1sq)' + ones(n1,1)*n1sq -2*X'*X;
	else
	  n2sq = sum(X2.^2);
	  n2 = size(X2,2);
	  D = (ones(n2,1)*n1sq)' + ones(n1,1)*n2sq -2*X'*X2;
	end;
	K = exp(-D/(2*sigma^2));  
	
end;

%CPU=cputime-t
